from xaal.schemas import devices
from xaal.lib import tools,Device

import logging
import tenacity

logger = logging.getLogger(__name__)

# we try twice within one second, if nothing works, just drop
TENACITY_RETRY=tenacity.stop_after_attempt(2) | tenacity.stop_after_delay(1)

def split_color(color):
    data = color.split('#')[-1]
    r = int(data[0:2],16)
    g = int(data[2:4],16)
    b = int(data[4:6],16)
    return (r,g,b)


class YeelightDev(object):
    def __init__(self,bulb,addr):
        logger.info('New device at %s : %s' % (bulb._ip,addr))
        self.bulb = bulb
        self.addr = addr
        self.dev = None
        self.setup()
        self.set_properties()

    def set_properties(self):
        self.dev.vendor_id = 'Yeelight'

    def setup(self):
        logger.warning('Please overide setup()')


class RGBW(YeelightDev):
    def setup(self):
        dev = devices.lamp_rgbw(self.addr)
        dev.methods['on']  = self.on
        dev.methods['off'] = self.off
        dev.methods['toggle'] = self.toggle
        dev.methods['dim'] = self.dim
        dev.methods['setRGB'] = self.set_color
        dev.methods['setWhite'] = self.set_white
        dev.info = 'RGBW / %s' % self.addr
        self.dev = dev

    @tenacity.retry(stop=TENACITY_RETRY)
    def on(self):
        self.bulb.turn_on()
        self.dev.attributes['light'] = True

    @tenacity.retry(stop=TENACITY_RETRY)
    def off(self):
        self.bulb.turn_off()
        self.dev.attributes['light'] = False

    @tenacity.retry(stop=TENACITY_RETRY)
    def toggle(self):
        self.bulb.toggle()
    
    def dim(self,_target):
        val = int(_target)
        self.on()
        self.bulb.set_brightness(val)

    def set_color(self,_target):
        print(_target)
        r,g,b = split_color(_target)
        self.on()
        self.bulb.set_rgb(r,g,b)
        self.dev.attributes['mode'] = 'RGB'
        self.dev.attributes['rgb'] = _target

    def set_white(self,_target):
        val = int(_target)
        self.on()
        self.bulb.set_color_temp(val)
        self.dev.attributes['mode'] = 'WHITE'
        self.dev.attributes['white_temperature'] = _target



